/*{{{  file header */

/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam,
                        The Netherlands.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/
/*
  $Id: tree.c,v 1.4 2000/05/15 15:42:24 markvdb Exp $  
 */

/*
 * The version of se is written by Mark van den Brand.
 */

/*}}}  */
/*{{{  includes */

#include <stdlib.h>

#include "se.h"   
#include "tree.h"
#include "sort.h"
#include "path.h"

/*}}}  */

/*{{{  ATerm getTreeSort(ATerm tree) */

ATerm getTreeSort(ATerm tree)
{
  ATerm t0;

  /*if(ATgetType(tree) == AT_LIST) {
    return tree;
    } else */

  if(ATgetType(tree) == AT_APPL) {  
    if(asfix_is_term(tree)) {
      return getTreeSort(ATgetArgument(tree, 6));
    }
    else if(asfix_is_lex(tree) || asfix_is_var(tree)) {
      return ATgetArgument(tree, 1);
    }
    else if(asfix_is_appl(tree)) {
      t0 = ATgetArgument(tree, 0);
      return ATgetArgument(t0,6);
    }
    else if(asfix_is_list(tree)) {
      return ATgetArgument(tree, 0);
    }
    else {
      return newSort("invalid");
    }
  }
  else {
    ATerror("getTreeSort: Tree sort not supported\n");
    return tree; 
  }
}

/*}}}  */
/*{{{  ATerm subtreeOfTreeGivenPath(ATerm tree, ATerm path) */

ATerm subtreeOfTreeGivenPath(ATerm tree, ATerm path)
{
  ATerm firstElement;

  if(ATgetType(tree) == AT_LIST) {
    if(isLeafPath(path)) {
      if(!ATisEmpty((ATermList)tree)) {
         return ATgetFirst((ATermList)tree);
      }
      else {
        return NULL;
      }
    }
    else if(isHeadZero(path)) {
      firstElement = ATgetFirst((ATermList)tree);
      if(ATgetType(firstElement) == AT_APPL) {
        if(asfix_is_appl(firstElement) || asfix_is_list(firstElement)) {
          return subtreeOfTreeGivenPath(
                   (ATerm)ATgetArgument((ATermAppl)firstElement, 2),
                   getTail(path));      
        }  
        else if(asfix_is_term(firstElement)) {
          ATerm newPath = getTail(path);
          newPath = decreaseHead(newPath);
          return subtreeOfTreeGivenPath(
                   ATgetArgument((ATermAppl)firstElement, 6),
                                 newPath);
        }
        else {
          return NULL;
        }
      }
      else {
        return NULL;
      }
    }
    else if(!isHeadZero(path)) {
      path = decreaseHead(path);
      return subtreeOfTreeGivenPath((ATerm)ATgetNext((ATermList)tree),
                                    path);
    }
    else {
      return NULL;
    }
  }     
  else if(ATgetType(tree) == AT_APPL) {
    if(isLeafPath(path)) {
      if(asfix_is_lex(tree) ||
         asfix_is_term(tree) ||
         asfix_is_var(tree) ||
         asfix_is_list(tree) ||
         asfix_is_whitespace(tree) ||
         asfix_is_appl(tree)) {
        return tree;
      }
      else {
        return NULL;
      }
    }
    else if(isHeadZero(path)) {
      if(asfix_is_appl(tree) || asfix_is_list(tree)) {
        return subtreeOfTreeGivenPath(
                 (ATerm)ATgetArgument((ATermAppl)tree, 2), getTail(path));
      }
      else if(asfix_is_term(tree)) {
        ATerm newPath = getTail(path);
        newPath = decreaseHead(newPath);
        return subtreeOfTreeGivenPath(ATgetArgument((ATermAppl)tree, 6),
                                      newPath);
      }    
      else {
        return NULL;
      }
    }
    else {
      return NULL;
    }
  }
  else {
    ATerror("In subtreeOfTreeGivenPath: Tree type not supported\n");
    return NULL;
  }
}                

/*}}}  */
/*{{{  ATerm updateTerm(ATerm tree, ATerm path, ATerm newTree) */

ATerm updateTerm(ATerm tree, ATerm path, ATerm newTree)
{
  ATerm firstElement, newFirstElement;
  ATerm newPath, tmpTree;
  ATerm t0, t1, t2, t3, t4, t5, t6, t7, t8;

  if(ATgetType(tree) == AT_LIST) {
    ATermList elements = (ATermList)tree;
    if(isHeadZero(path)) {
      firstElement = ATgetFirst(elements);
      elements = ATgetNext(elements);
      newFirstElement = updateTerm(firstElement, path, newTree);
      elements = ATinsert(elements, newFirstElement);

      /* Something with whitespace */

      return (ATerm)elements;
    }
    else if(isLeafPath(path)) {
      elements = ATgetNext(elements);
      elements = ATinsert(elements, newTree);
      return (ATerm)elements;
    }
    else {
      newPath = decreaseHead(path);
      firstElement = ATgetFirst(elements);
      elements = ATgetNext(elements);
      tmpTree = updateTerm((ATerm)elements, newPath, newTree);

      /* Something with whitespace */

      if(ATgetType(tmpTree) == AT_LIST) {
        return (ATerm)ATinsert((ATermList)tmpTree, firstElement);
      }
      else {
        return (ATerm)ATmakeList2(firstElement, tmpTree);
      }
    }
  }     
  else if(ATgetType(tree) == AT_APPL) {
    if(isLeafPath(path)) {
      return newTree;
    }
    else if(isHeadZero(path)) {
      if(asfix_is_appl(tree)) {
        t0 = ATgetArgument((ATermAppl)tree, 0);
        t1 = ATgetArgument((ATermAppl)tree, 1);    
        t2 = updateTerm((ATerm)ATgetArgument((ATermAppl)tree, 2),
                        getTail(path), newTree);
        return (ATerm)ATmakeAppl3(symbol_appl,t0,t1,t2);
      }
      if(asfix_is_list(tree)) {
        t0 = ATgetArgument((ATermAppl)tree, 0);
        t1 = ATgetArgument((ATermAppl)tree, 1);    
        t2 = updateTerm((ATerm)ATgetArgument((ATermAppl)tree, 2),
                        getTail(path), newTree);
        return (ATerm)ATmakeAppl3(af_symbol_list,t0,t1,t2);
      }
      else if(asfix_is_term(tree)) {
        ATerm newPath = getTail(path);
        newPath = decreaseHead(newPath);
        t0 = ATgetArgument((ATermAppl)tree, 0);
        t1 = ATgetArgument((ATermAppl)tree, 1);
        t2 = ATgetArgument((ATermAppl)tree, 2);
        t3 = ATgetArgument((ATermAppl)tree, 3);
        t4 = ATgetArgument((ATermAppl)tree, 4);
        t5 = ATgetArgument((ATermAppl)tree, 5);
        t6 = updateTerm((ATerm)ATgetArgument((ATermAppl)tree, 6),
                        newPath, newTree);  
        t7 = ATgetArgument((ATermAppl)tree, 7);
        t8 = ATgetArgument((ATermAppl)tree, 8);

        /* Something with whitespace */

        return (ATerm)ATmakeAppl(symbol_term,t0,t1,t2,t3,t4,t5,t6,t7,t8);    
      }    
    }
    else {
      ATerror("updateTerm failed\n");
      return tree;
    }
  }
  else {
    ATerror("In updateTerm: Tree type not supported\n");
    return tree;
  }
}                

/*}}}  */
